import 'dart:async';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

typedef MRZScannerCallback = void Function(String result);

class Mrzflutterplugin {
  static const MethodChannel _channel = const MethodChannel('mrzflutterplugin');

  // Initiates the scanner view
  static Future<String> get startScanner async {
    final String scannerResult = await _channel.invokeMethod("startScanner");
    return scannerResult;
  }

  // Initiates the scanner view with continuous scanning enabled
  // After successful scan, the scanner view will not be dismissed or paused.
  // To close the scanner, call closeScanner();
  //
  // [ignoreDuplicates] is used to specify whether the scanner should ignore
  // the successful scan if the document number is identical to the previous
  // successful scan
  static void startContinuousScanner(MRZScannerCallback callback,
      [bool ignoreDuplicates = true]) {
    _channel.invokeMethod("startContinuousScanner", ignoreDuplicates);
    _channel.setMethodCallHandler((call) {
      if (call.method == "onScannerResult") callback.call(call.arguments);
    });
  }

  // Initiates the scanner view with specific frame. The values represent percentages relative to the screen size [0...100]
  static Future<String> startPartialViewScanner(
      int x, int y, int width, int height) async {
    final String scannerResult = await _channel
        .invokeMethod("startPartialViewScanner", [x, y, width, height]);
    return scannerResult;
  }

  // Initiates the scanner view
  static Future<String> get captureIdImage async {
    final String scannerResult = await _channel.invokeMethod("captureIdImage");
    return scannerResult;
  }

  // Register with the licence key provided to remove the asterisks (*) from the result.
  static void registerWithLicenceKey(String licenceKey) {
    _channel.invokeMethod("registerWithLicenceKey", licenceKey);
  }

  // Specify whether the scanner should detect and return result for passports.
  static void setPassportActive(bool active) {
    _channel.invokeMethod("setPassportActive", active);
  }

  // Specify whether the scanner should detect and return result for IDs.
  static void setIDActive(bool active) {
    _channel.invokeMethod("setIDActive", active);
  }

  // Specify whether the scanner should detect and return result for Visas.
  static void setVisaActive(bool active) {
    _channel.invokeMethod("setVisaActive", active);
  }

  // Set the date format in which the parsed dates are formatted.
  static void setDateFormat(String dateFormat) {
    _channel.invokeMethod("setDateFormat", dateFormat);
  }

  // Trigger an image picker.
  static Future<String> get scanFromGallery async {
    final String scannerResult = await _channel.invokeMethod("scanFromGallery");
    return scannerResult;
  }

  // Returns the current SDK Version
  static Future<String> get getSdkVersion async {
    final String sdkVersion = await _channel.invokeMethod("getSdkVersion");
    return sdkVersion;
  }

  // Resume scanning after a successful scan.
  static void resumeScanner() {
    _channel.invokeMethod("resumeScanner");
  }

  // Stop and close the scanner.
  static void closeScanner() {
    _channel.invokeMethod("closeScanner");
  }

  // Specify whether the device should vibrate when the scanner performs a successful scan. Default value: true.
  static void setVibrateOnSuccessfulScan(bool vibrate) {
    _channel.invokeMethod("setVibrateOnSuccessfulScan", vibrate);
  }

  // Specify whether the device should use the front-facing camera for scanning. Default value: false.
  static void setUseFrontCamera(bool enabled) {
    _channel.invokeMethod("setUseFrontCamera", enabled);
  }

  // Warn if the camera does not meet the minimum requirements. Default value: true.
  static void setWarnIncompatibleCamera(bool enabled) {
    _channel.invokeMethod("setWarnIncompatibleCamera", enabled);
  }

}

class MRZScanner extends StatelessWidget {

  final bool idActive;
  final bool passportActive;
  final bool visaActive;
  final String licenceKey;

  MRZScanner({
    @required this.licenceKey,
    @required this.idActive,
    @required this.passportActive,
    @required this.visaActive,
  });

  @override
  Widget build(BuildContext context) {

  }
}
